%global frr_libdir %{_libexecdir}/frr
%global _hardened_build 1

%bcond_with grpc

Summary:        Routing daemon
Name:           frr
Version:        10.0.3
Release:        1%{?dist}
License:        GPLv2+
URL:            http://www.frrouting.org
Source0:        https://github.com/FRRouting/frr/archive/refs/tags/%{name}-%{name}-%{version}.tar.gz
Source1:        %{name}-tmpfiles.conf
Source2:        %{name}-sysusers.conf
Source3:        frr.fc
Source4:        frr.te
Source5:        frr.if

Patch0001:      CVE-2024-44070.patch

Patch3000:      0002-enable-openssl.patch
Patch3001:      0003-disable-eigrp-crypto.patch

BuildRequires:  autoconf automake bison c-ares-devel flex gcc
BuildRequires:  gcc-c++ git-core groff json-c-devel libcap-devel
BuildRequires:  libtool libyang-devel make ncurses ncurses-devel net-snmp-devel
BuildRequires:  pam-devel patch perl-XML-LibXML perl-generators python3-devel
BuildRequires:  python3-pytest  python3-sphinx readline-devel systemd-devel
BuildRequires:  systemd-rpm-macros texinfo protobuf-c-devel
%if %{with grpc}
BuildRequires:  grpc-devel grpc-plugins
%endif

Requires:       ncurses net-snmp
Requires(post): hostname
%{?sysusers_requires_compat}
%{?systemd_requires}
Requires: (%{name}-selinux = %{version}-%{release} if selinux-policy-targeted)
Provides:       routingdaemon = %{version}-%{release}

%description
FRRouting is free software that manages TCP/IP based routing protocols. It takes a multi-server and
multi-threaded approach to resolve the current complexity of the Internet.

FRRouting supports BGP4, OSPFv2, OSPFv3, ISIS, RIP, RIPng, PIM, NHRP, PBR, EIGRP and BFD.

FRRouting is a fork of Quagga.

%package selinux
Summary:    Selinux policy for FRR
BuildRequires:  selinux-policy-devel
Requires:       selinux-policy-targeted
Requires(post): selinux-policy-targeted
%{?selinux_requires}
BuildArch:    noarch

%description selinux
SELinux policy modules for FRR package

%prep
%autosetup -n %{name}-%{name}-%{version} -p1 -Sgit
mkdir selinux
cp -p %{SOURCE3} %{SOURCE4} %{SOURCE5} selinux
# C++14 or later needed for abseil-cpp 20230125; string_view needs C++17:
sed -r -i 's/(AX_CXX_COMPILE_STDCXX\(\[)11(\])/\117\2/' configure.ac

%build
autoreconf -ivf

%configure \
    --sbindir=%{frr_libdir} \
    --sysconfdir=%{_sysconfdir}/frr \
    --libdir=%{_libdir}/frr \
    --libexecdir=%{_libexecdir}/frr \
    --localstatedir=%{_localstatedir}/run/frr \
    --enable-multipath=64 \
    --enable-vtysh=yes \
    --disable-ospfclient \
    --disable-ospfapi \
    --enable-snmp=agentx \
    --enable-user=frr \
    --enable-group=frr \
    --enable-vty-group=frrvty \
    --enable-rtadv \
    --disable-exampledir \
    --enable-systemd=yes \
    --enable-static=no \
    --disable-ldpd \
    --disable-babeld \
    --with-moduledir=%{_libdir}/frr/modules \
    --with-crypto=openssl \
    --enable-fpm \
    --enable-irdp \
%if %{with grpc}
    --enable-grpc
%endif

%make_build MAKEINFO="makeinfo --no-split" PYTHON=%{__python3}

%make_build -C doc info

make -C selinux -f %{_datadir}/selinux/devel/Makefile %{name}.pp
bzip2 -9 selinux/%{name}.pp

%install
mkdir -p %{buildroot}%{_sysconfdir}/{frr,rc.d/init.d,sysconfig,logrotate.d,pam.d,default} \
         %{buildroot}%{_localstatedir}/log/frr %{buildroot}%{_infodir} \
         %{buildroot}%{_unitdir}

mkdir -p -m 0755 %{buildroot}%{_libdir}/frr
mkdir -p %{buildroot}%{_tmpfilesdir}
mkdir -p %{buildroot}%{_sysusersdir}

%make_install

rm -rf %{buildroot}%{_infodir}/dir

install -p -m 644 %{SOURCE1} %{buildroot}%{_tmpfilesdir}/%{name}.conf
install -p -m 644 %{SOURCE2} %{buildroot}%{_sysusersdir}/%{name}.conf
install -p -m 644 tools/etc/frr/daemons %{buildroot}%{_sysconfdir}/frr/daemons
install -p -m 644 tools/frr.service %{buildroot}%{_unitdir}/frr.service
install -p -m 755 tools/frrinit.sh %{buildroot}%{frr_libdir}/frr
install -p -m 755 tools/frrcommon.sh %{buildroot}%{frr_libdir}/frrcommon.sh
install -p -m 755 tools/watchfrr.sh %{buildroot}%{frr_libdir}/watchfrr.sh

install -p -m 644 redhat/frr.logrotate %{buildroot}%{_sysconfdir}/logrotate.d/frr
install -p -m 644 redhat/frr.pam %{buildroot}%{_sysconfdir}/pam.d/frr
install -d -m 775 %{buildroot}/run/frr

install -D -m 644 selinux/%{name}.pp.bz2 \
    %{buildroot}%{_datadir}/selinux/packages/targeted/%{name}.pp.bz2
install -D -m 644 selinux/%{name}.if %{buildroot}%{_datadir}/selinux/devel/include/distributed/%{name}.if

find %{buildroot} -type f -name "*.la" -delete -print

rm %{buildroot}%{_libdir}/frr/*.so
rm -r %{buildroot}%{_includedir}/frr/

%check
rm tests/lib/*grpc*
%make_build check PYTHON=%{__python3}

%pre
%sysusers_create_compat %{SOURCE2}

%post
%systemd_post frr.service

if [ ! -e %{_sysconfdir}/frr/frr.conf ]; then
    echo "hostname `hostname`" > %{_sysconfdir}/frr/frr.conf
    chown frr:frr %{_sysconfdir}/frr/frr.conf
    chmod 640 %{_sysconfdir}/frr/frr.conf
fi

if [ ! -e %{_sysconfdir}/frr/vtysh.conf ]; then
    touch %{_sysconfdir}/frr/vtysh.conf
    chmod 640 %{_sysconfdir}/frr/vtysh.conf
    chown frr:frrvty %{_sysconfdir}/frr/vtysh.conf
fi

%postun
%systemd_postun_with_restart frr.service

%preun
%systemd_preun frr.service

%pre selinux
%selinux_relabel_pre -s targeted

%post selinux
%selinux_modules_install -s targeted %{_datadir}/selinux/packages/targeted/%{name}.pp.bz2
%selinux_relabel_post -s targeted
if [ $1 == 2 ]; then
    %{_sbindir}/restorecon -R /var/tmp/frr &> /dev/null
    %{_sbindir}/restorecon -R /var/run/frr &> /dev/null
fi

%postun selinux
if [ $1 -eq 0 ]; then
    %selinux_modules_uninstall -s targeted %{name}
    %selinux_relabel_post -s targeted
fi

%files
%license COPYING
%doc doc/mpls
%dir %attr(750,frr,frr) %{_sysconfdir}/frr
%dir %attr(755,frr,frr) %{_localstatedir}/log/frr
%dir %attr(755,frr,frr) /run/frr
%config(noreplace) %attr(644,root,root) %{_sysconfdir}/logrotate.d/frr
%config(noreplace) %attr(644,frr,frr) %{_sysconfdir}/frr/daemons
%config(noreplace) %{_sysconfdir}/pam.d/frr
%{_tmpfilesdir}/%{name}.conf
%{_sysusersdir}/%{name}.conf
%{_unitdir}/*.service
%{_infodir}/*info*
%dir %{frr_libdir}/
%{frr_libdir}/*
%{_bindir}/mtracebis
%{_bindir}/vtysh
%dir %{_libdir}/frr
%{_libdir}/frr/*.so.*
%dir %{_libdir}/frr/modules
%{_libdir}/frr/modules/*
%dir %{_datadir}/yang
%{_datadir}/yang/*.yang
%{_mandir}/man1/frr.1*
%{_mandir}/man1/vtysh.1*
%{_mandir}/man8/frr-*.8*
%{_mandir}/man8/mtracebis.8*

%files selinux
%{_datadir}/selinux/packages/targeted/%{name}.pp.*
%{_datadir}/selinux/devel/include/distributed/%{name}.if
%ghost %verify(not md5 size mode mtime) %{_sharedstatedir}/selinux/targeted/active/modules/200/%{name}

%changelog
* Tue Apr 22 2025 xiaoyunzhao <xiaoyunzhao@tencent.com> - 10.0.3-1
- [Type] security
- [Desc] Upgrade to 10.0.3 and fix CVE-2024-55553

* Thu Sep 26 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 9.0.3-2
- Rebuilt for clarifying the packages requirement in BaseOS and AppStream

* Mon Sep 23 2024 kianli <kianli@tencent.com> - 9.0.3-1
- [Type] security
- [Desc] Upgrade to 9.0.3 and fix CVE-2024-31948, CVE-2024-31949, CVE-2024-44070

* Fri Aug 16 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 9.0.1-4
- Rebuilt for loongarch release

* Tue Jun 04 2024 kianli <kianli@tencent.com> - 9.0.1-3
- [Type] security
- [DESC] fix CVE-2024-31950, CVE-2024-31951, CVE-2024-34088

* Mon Dec 11 2023 kianli <kianli@tencent.com> - 9.0.1-2
- Security fix for CVE-2023-47234 and CVE-2023-47235

* Fri Nov 10 2023 kianli <kianli@tencent.com> - 9.0.1-1
- Upgrade to 9.0.1
- Security fix for CVE-2023-38406
- Security fix for CVE-2023-38407
- Security fix for CVE-2023-38802
- Security fix for CVE-2023-41359
- Security fix for CVE-2023-41909
- Security fix for CVE-2023-41360
- Security fix for CVE-2023-41361
- Security fix for CVE-2023-41358
- Security fix for CVE-2023-46752
- Security fix for CVE-2023-46753

* Wed Sep 20 2023 Miaojun Dong <zoedong@tencent.com> - 8.4-6
- Disable experimental grpc, currently not compatible with protobuf-24.2 and grpc-1.56.2

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 8.4-5
- Rebuilt for OpenCloudOS Stream 23.09

* Thu Sep 07 2023 jackeyji <jackeyji@tencent.com> - 8.4-4
- Rebuilt for protobuf 3.21.12

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 8.4-3
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 8.4-2
- Rebuilt for OpenCloudOS Stream 23

* Wed Mar 29 2023 Shuo Wang <abushwang@tencent.com> - 8.4-1
- initial build
